/* 
   portblocker.c

   Copyright (C) 2003-2004 FreeIPMI Core Team

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/socket.h>
#include <string.h>
#include <sys/resource.h>
#include <sys/ioctl.h>

#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <signal.h>
#include <errno.h>

char pidfile[128];

void 
tidy_up (int mysig)
{
  //fprintf (stderr, "shutting down\n");
  unlink (pidfile);
  
  exit (0);
}

int 
daemon_init (char *pidfile)
{
  pid_t pid, thispid;
  FILE  *pidout;
  int fd;
  
  if ((pid = fork ()) < 0) 
    return -1;
  else 
    {
      if (pid != 0) 
	exit (0);   /* exit the original foreground call */
    }
  
  /* Only the child process continues */
  setsid ();
  chdir ("/");
  umask (0);
  
  //errout = fopen("/var/log/usb_perm.log", "a");
  //  fprintf((char *)errout,"----------\n");
  
  thispid = getpid ();
  pidout = fopen (pidfile, "w");
  if (pidout == NULL)
    return -1;
  
  fprintf (pidout, "%d\n", thispid);
  fclose (pidout);
  
  if ((fd = open ("/dev/tty", O_RDWR)) != -1)
    {
      ioctl (fd, TIOCNOTTY, (char *) 0);
      close (fd);
    }
  
  return 0;
}

struct sockaddr_in myaddr;         // server address
int listener = 0;                  // listening socket descriptor

void 
init_server (int port)
{
  int yes = 1;        // for setsockopt() SO_REUSEADDR, below
  
  // get the listener
  if ((listener = socket (AF_INET, SOCK_STREAM, 0)) == -1)
    {
      perror ("socket()");
      exit (-1);
    }
  
  // lose the pesky "address already in use" error message
  if (setsockopt (listener, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof (int)) 
      == -1)
    {
      perror("setsockopt()");
      exit (-1);
    }
  
  // bind
  myaddr.sin_family = AF_INET;
  myaddr.sin_addr.s_addr = INADDR_ANY;
  myaddr.sin_port = htons (port);
  memset (&(myaddr.sin_zero), '\0', 8);
  if (bind (listener, (struct sockaddr *) &myaddr, sizeof (myaddr)) == -1)
    {
      perror ("bind()");
      exit (-1);
    }
  
  // listen
  if (listen (listener, 10) == -1)
    {
      perror ("listen()");
      exit (-1);
    }
}

int 
get_server_socket ()
{
  return listener;
}

int 
accept_connection (int listener)
{
  struct sockaddr_in remoteaddr; // client address
  int addrlen;
  int newfd;
  
  // handle new connections
  addrlen = sizeof (remoteaddr);
  newfd = accept (listener, (struct sockaddr *) &remoteaddr, 
		  &addrlen);
  
  if (newfd == -1)
    {
      perror ("portblocker");
      return -1;
    }
  
  write (newfd, "DO NOT CONNECT TO ME\n", strlen ("DO NOT CONNECT TO ME\n"));
  close (newfd);
  
  return 0;
}

int 
main (int argc, char **argv)
{
  int server_socket;
  
  if (argc != 2)
    {
      fprintf (stderr, "usage: %s <PORTNUMBER>\n", program_invocation_short_name);
      exit (-1);
    }
  
  init_server (atoi (argv[1]));
  
  sprintf (pidfile, "/var/run/portblocker%d.pid", atoi (argv[1]));
  
  if (daemon_init (pidfile))
    {
      fprintf (stderr, "%s: daemon init failed\n",
	       program_invocation_short_name);
      exit (-1);
    }
  
  signal (SIGTERM, tidy_up);
  signal (SIGINT, tidy_up);
  //signal (SIGTSTP, tidy_up);
  
  //fprintf (stderr, "daemon started.\n");
  
  server_socket = get_server_socket ();
  
  while (1) 
    accept_connection (server_socket);
  
  return 0;
}

